
#pragma OPENCL EXTENSION cl_khr_fp64 : enable
__kernel void prefixSum(__global int *input,
						__global int *output,
						__global int *temp,
						const unsigned int inputSize,
						const unsigned int height)
{
    // Get our global thread ID
    int id = get_global_id(0);

	// Copy input to the temporary array (which represents a tree)
	temp[id*2] = input[id*2];
	temp[id*2+1] = input[id*2+1];

	// Perform the up-sweep phase
	for (int n = 1; n < height; n++) {
		// All threads ned to be done with the last level of the tree before we go to the next one
		barrier(CLK_GLOBAL_MEM_FENCE);
		// Add the left and right subnode to the current node assigned to this thread
		temp[n*inputSize + id] = temp[(n - 1)*inputSize + id * 2] + temp[(n - 1)*inputSize + id * 2 + 1];
	}

	// Perform the down-sweep phase
	// Set root to 0
	temp[(height-1)*inputSize + id] = 0;
	for (int n = height-1; n > 0; n--) {
		// All threads ned to be done with the last level of the tree before we go to the next one
		barrier(CLK_GLOBAL_MEM_FENCE);
		temp[(n - 1)*inputSize + id * 2 + 1] = temp[(n - 1)*inputSize + id * 2] + temp[n*inputSize + id];
		temp[(n - 1)*inputSize + id * 2] = temp[n*inputSize + id];
		// right: temp[(n-1)*inputSize + id*2 + 1]
		// left: temp[(n-1)*inputSize + id*2]
		// root: temp[n*inputSize + id]
	}
	// Copy the leafs of the tree (located in the temp array) to the output array
	barrier(CLK_GLOBAL_MEM_FENCE);
	output[id*2] = temp[id*2];
	output[id * 2 + 1] = temp[id * 2 + 1];
}


