#pragma once
#include "GPUSimple1D.h"
#include "GPUSimple2D.h"
#include "GPUAdvanced.h"
#include <fstream>
using namespace std;

void testPerformanceSimple1D() {
	ofstream resultFile;
	resultFile.open("results.csv", ios::out);

	if (!resultFile.is_open()) {
		printf("Failed to open results file!");
		return;
	}
	for (int inputSize = 2; inputSize <= 1024; inputSize = inputSize*2) {  // Alternatively use 'inputSize = inputSize++' to go through all sizes
		printf("InputSize: %d\n", inputSize);
		int *input = generateIncreasingInput(inputSize);
		Result result = executeSimple1D(inputSize, input);
		resultFile << result.executionTime << endl;
		delete[] result.output;
	}
}

void testSimple1D() {
	int inputSize = 8;
	executeSimple1D(inputSize, generateIncreasingInput(inputSize));
}

void testPerformanceSimple2D() {
	ofstream resultFile;
	resultFile.open("results.csv", ios::out);

	if (!resultFile.is_open()) {
		printf("Failed to open results file!");
		return;
	}
	for (int inputSize = 2; inputSize <= 1024; inputSize = inputSize*2) {  // Alternatively use 'inputSize = inputSize++' to go through all sizes
		printf("InputSize: %d\n", inputSize);
		int *input = generateIncreasingInput(inputSize);
		Result result = executeSimple2D(inputSize, input);
		resultFile << inputSize << ", " << result.executionTime << endl;
		delete[] result.output;
	}
}

void testSimple2D() {
	int inputSize = 8;
	executeSimple2D(inputSize, generateIncreasingInput(inputSize));
}

void testPerformanceAdvanced() {
	ofstream resultFile;
	resultFile.open("results.csv", ios::out);

	if (!resultFile.is_open()) {
		printf("Failed to open results file!");
		return;
	}
	for (int inputSize = 2; inputSize <= 1024; inputSize = inputSize*2) {  // Alternatively use 'inputSize = inputSize++' to go through all sizes
		printf("InputSize: %d\n", inputSize);
		int *input = generateIncreasingInput(inputSize);
		Result result = executeAdvanced(inputSize, input);
		resultFile << inputSize << ", " << result.executionTime << endl;
		delete[] result.output;
	}
}

void testAdvanced() {
	int inputSize = 16;
	executeAdvanced(inputSize, generateIncreasingInput(inputSize));
}

int main() {
	//// Enable one or more of the method calls below
	//// Ones without 'Performance' will do a single run with the input size defined in the method
	//// Ones with 'Performance' will do a sequence of runs with input sizes determined in the methods
	
	//testSimple1D();
	//testPerformanceSimple1D();

	//testSimple2D();
	//testPerformanceSimple2D();

	//testAdvanced();
	//testPerformanceAdvanced();

	//sclReleaseClHard(getHardware());
	system("PAUSE");
	return 0;
}

